# -*- coding: utf-8 -*-
"""
app.utils
~~~~~~~~~

This module implements utility functions.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import base64
import datetime
import io
import sqlalchemy as sa


class InventoryError(Exception):
    """InventoryError class

    Exception used to pass useful information to the client side (API or AJAX)
    """
    status_code = 400

    def __init__(self, message, status_code=None, payload=None):
        super().__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload

    def to_dict(self):
        rv = dict(self.payload or ())
        rv['message'] = self.message
        return rv

    def __str__(self):
        return str(self.to_dict())


def image_to_base64(img, format='PNG'):
    """Convert a Pillow image to a base64 string

    :param img: Pillow image
    :param format: format of the image to use
    :returns str: image as base64 string
    """
    buf = io.BytesIO()
    img.save(buf, format=format)
    return base64.b64encode(buf.getvalue()).decode('ascii')


def format_field(field):
    """Format the given field to a string or None"""
    if field is None:
        return None
    if isinstance(field, datetime.datetime):
        return field.strftime('%Y-%m-%d %H:%M')
    return str(field)


def convert_to_model(item, model):
    """Convert item to an instance of model

    Allow to convert a string to an instance of model
    Raise an exception if the given name is not found

    :returns: an instance of model
    """
    if item is None:
        return None
    if not isinstance(item, model):
        instance = model.query.filter_by(name=item).first()
        if instance is None:
            raise InventoryError(f'{item} is not a valid {model.__name__.lower()}')
        return instance
    return item


def attribute_to_string(value):
    """Return the attribute as a string

    If the attribute is defined in the schema as multi valued
    then the attribute value is returned as a list
    See http://ldap3.readthedocs.io/tutorial_searches.html#entries-retrieval

    This function returns the first item of the list if it's a list

    :param value: string or list
    :returns: string
    """
    if isinstance(value, list):
        return value[0]
    else:
        return value


def get_choices(iterable, allow_blank=False, allow_null=False):
    """Return a list of (value, label)"""
    choices = []
    if allow_blank:
        choices = [('', '')]
    if allow_null:
        choices.append(('null', 'not set'))
    choices.extend([(val, val) for val in iterable])
    return choices


def get_query(query, args):
    """Retrieve the query from the arguments

    :param query: sqlalchemy base query
    :param MultiDict args: args from a request
    :returns: query filtered by the arguments
    """
    if args:
        kwargs = args.to_dict()
        try:
            query = query.filter_by(**kwargs)
        except (sa.exc.InvalidRequestError, AttributeError) as e:
            raise InventoryError('Invalid query arguments', status_code=422)
    return query
