import os
import logging
import time
import click
import daiquiri
import requests
import serial
import yaml
from collections import namedtuple


QRCode = namedtuple('QRCode', 'table id name')
daiquiri.setup(level=logging.DEBUG)
# logger = daiquiri.getLogger(__name__)
logger = daiquiri.getLogger('pyscan')


def get_config(filename):
    with open(filename) as f:
        config = yaml.load(f)
    return config


def get_token(url, username):
    password = click.prompt('Please enter your password', hide_input=True)
    payload = {'username': username, 'password': password}
    r = requests.post(url + '/login', json=payload)
    r.raise_for_status()
    logger.info(f'Successfully logged in to {url}!')
    return r.json()['access_token']


def save_item(url, token, item):
    headers = {'Authorization': f'Bearer {token}'}
    r = requests.post(url + '/items', json=item, headers=headers)
    r.raise_for_status()


def save_items(url, token, items):
    for item in items:
        try:
            save_item(url, token, item)
        except requests.exceptions.HTTPError as err:
            logger.warning(err)
        else:
            logger.info(f'Item saved: {item}')


class Scanner:

    def __init__(self, device, url, token):
        self.device = device
        self.url = url
        self.token = token
        self.items = []
        self._manufacturer = None
        self._model = None
        self._location = None
        self._status = None

    def process_qrcode(self, qrcode):
        if qrcode.table in ('manufacturer', 'model', 'location', 'status'):
            logger.info(f'Setting {qrcode.table} to {qrcode.name}')
            setattr(self, '_' + qrcode.table, qrcode.name)
        elif qrcode.table == 'action':
            if qrcode.name == 'Discard last':
                try:
                    item = self.items.pop()
                except IndexError:
                    logger.warning('No item to remove!')
                else:
                    logger.info(f'Item {item} discarded')
            elif qrcode.name == 'Clear attributes':
                logger.info('Clearing manufacturer/model/location/status')
                self._manufacturer = None
                self._model = None
                self._location = None
                self._status = None
            elif qrcode.name == 'Save to db':
                logger.info('Saving all items to database')
                save_items(self.url, self.token, self.items)
                self.items = []
            else:
                logger.warning(f'Unknown action: {qrcode.name}')
        else:
            logger.warning(f'Unknown table: {qrcode.table}')

    def process_serial_number(self, serial_number):
        item = {'serial_number': serial_number,
                'manufacturer': self._manufacturer,
                'model': self._model,
                'location': self._location,
                'status': self._status,
                }
        logger.info(f'Creating item: {item}')
        self.items.append(item)

    def parse(self, line):
        logger.debug(line)
        try:
            qrcode = QRCode(*line.split(','))
        except TypeError:
            # Assume this is a serial number
            self.process_serial_number(line)
        else:
            self.process_qrcode(qrcode)

    def run(self):
        logger.debug(self.device)
        ser = serial.Serial(self.device, 115200)
        while (True):
            if ser.in_waiting > 0:
                line = ser.readline()
                self.parse(line.decode('ascii').strip())
            time.sleep(0.1)


@click.command()
@click.option('--conf', '-c', default='~/.pyscan.yml',
              help='Configuration file [default: "~/pyscan.yml"]')
def scan(conf):
    filename = os.path.expanduser(conf)
    config = get_config(filename)
    url = config['url']
    username = config['username']
    token = get_token(url, username)
    scanner = Scanner(config['device'], url, token)
    scanner.run()


if __name__ == '__main__':
    scan()
